@@ -85,9 +85,21 @@ class DocumentSavedState(object):
                 return True
         return False
 
-    def __len__(self):
-        # for exps like "if document._saved_saved_state: ..."
-        return bool(self.storage and self.key)
+    def __hash__(self):
+        """
+        Storage and primary key together make the hash; document class doesn't
+        matter.
+
+        Raises `TypeError` if storage or primary key is not defined.
+        """
+        if not all(self.storage, self.key):
+            raise TypeError('Document is unhashable: storage or primary key '
+                            'is not defined')
+        return hash(self.storage) | hash(self.key)
+
+    def __nonzero__(self):
+        # for exps like "if document._saved_state: ..."
+        return any(self.storage or self.key)
 
     def clone(self):
         c = type(self)()
@@ -268,6 +280,9 @@ class Document(ProxyDict):
             # the DotDict stuff
             return super(Document, self).__getitem__(key)
 
+    def __hash__(self):
+        return hash(self._saved_state)
+
     def __init__(self, **kw):
         if self.__class__ == Document:
             raise NotImplementedError('Document must be subclassed')
@@ -629,11 +644,15 @@ class Document(ProxyDict):
 
         # prepare (validate) properties defined in the model
         # XXX only flat structure is currently supported:
-        for name in self.meta.structure:
-            value = self._data.get(name)
-            logging.debug('converting %s (%s) -> %s' % (name, repr(value),
-                          repr(storage.value_to_db(value))))
-            data[name] = storage.value_to_db(value)
+        if self.meta.structure:
+            for name in self.meta.structure:
+                value = self._data.get(name)
+                logging.debug('converting %s (%s) -> %s' % (name, repr(value),
+                              repr(storage.value_to_db(value))))
+                data[name] = storage.value_to_db(value)
+        else:
+            # free-form document
+            data.update(self._data)
 
         # TODO: make sure we don't overwrite any attrs that could be added to this
         # document meanwhile. The chances are rather high because the same document
